"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = buildHtml;

var _gulp = require("gulp");

var _gulpPlumber = _interopRequireDefault(require("gulp-plumber"));

var _gulpIf = _interopRequireDefault(require("gulp-if"));

var _gulpSort = _interopRequireDefault(require("gulp-sort"));

var _gulpHb = _interopRequireDefault(require("gulp-hb"));

var _gulpData = _interopRequireDefault(require("gulp-data"));

var _gulpHtmlmin = _interopRequireDefault(require("gulp-htmlmin"));

var _gulpRename = _interopRequireDefault(require("gulp-rename"));

var _lodash = _interopRequireDefault(require("lodash.clonedeep"));

var _handlebars = _interopRequireDefault(require("handlebars"));

var _handlebarsLayouts = _interopRequireDefault(require("handlebars-layouts"));

var helpers = _interopRequireWildcard(require("@hidoo/handlebars-helpers"));

var _gulpUtilErrorHandler = _interopRequireDefault(require("@hidoo/gulp-util-error-handler"));

var _dataFrom = require("@hidoo/data-from");

var _pathDepth = _interopRequireDefault(require("./pathDepth"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * task default options.
 * @type {Object}
 */
const DEFAULT_OPTIONS = {
  // task name (set displayName)
  name: 'build:html',
  // source path (required)
  src: null,
  // destination path (required)
  dest: null,
  // destination extname
  extname: '.html',
  // Handlebars partials files glob pattern
  partials: '',
  // Handlebars layouts files glob pattern
  layouts: '',
  // Handlebars helpers files glob pattern
  helpers: '',
  // data files glob pattern
  data: '',
  // compress file or not
  compress: false,
  // compress options (using htmlmin)
  compressOptions: {
    caseSensitive: true,
    collapseWhitespace: true,
    conservativeCollapse: true,
    preserveLineBreaks: true,
    ignoreCustomFragments: [// php start end tags
    /<\?[\s\S]*?\?>/, // cms tags
    /<\/?mt:?[\s\S]*?>/i, /<\$mt:?[\s\S]*?\$>/i]
  },

  /**
   * additional process after data files parsed
   * @param {Object} context data
   * @return {Object}
   */
  onFilesParsed(context) {
    return context;
  },

  /**
   * additional process after front matter parsed
   * @param {Object} context data
   * @return {Object}
   */
  onFrontMatterParsed(context) {
    return context;
  },

  // out log or not
  verbose: false
};
/**
 * task name.
 * @type {String}
 */

const TASK_NAME = 'gulp-task-build-html-handlebars';
/**
 * get context data from files
 * @param {String} pattern glob pattern
 * @param {Object} options options
 * @param {HandlebarsEnvironment} options.handlebars Handlrbars instance
 * @param {Function} options.onParsed process after files parsed
 * @param {Boolean} options.verbose out log or not
 * @return {Object}
 */

function getContextFromFiles(pattern, options = {}) {
  if (typeof pattern !== 'string' || pattern === '') {
    return {};
  }

  const {
    handlebars,
    onParsed,
    verbose
  } = options;
  let context = {};

  try {
    context = (0, _dataFrom.fromFiles)(pattern, {
      handlebars
    });

    if (typeof onParsed === 'function') {
      return onParsed(context);
    }

    return context;
  } catch (error) {
    if (verbose) {
      (0, _gulpUtilErrorHandler.default)(_objectSpread({
        plugin: TASK_NAME
      }, error));
    }

    return context;
  }
}
/**
 * get context data from front matter
 * + add path information of files included in Vinyl
 * @param {Vinyl} file Vinyl file
 * @param {Object} options options
 * @param {Object} options.context additinal data
 * @param {HandlebarsEnvironment} options.handlebars Handlrbars instance
 * @param {Function} options.onParsed process after front matter parsed
 * @return {Object}
 */


function getContextFromFrontMatter(file, options = {}) {
  const {
    contents,
    relative,
    basename,
    extname
  } = file,
        {
    context,
    handlebars,
    onParsed
  } = options,
        {
    body,
    attributes
  } = (0, _dataFrom.fromFrontMatter)(contents.toString(), {
    context,
    handlebars
  }),
        path = {
    depth: (0, _pathDepth.default)(relative),
    relative,
    basename,
    extname
  };

  if (typeof onParsed === 'function') {
    const processedAttributes = onParsed(attributes);
    return {
      body,
      attributes: _objectSpread({}, processedAttributes, {
        path
      })
    };
  }

  return {
    body,
    attributes: _objectSpread({}, attributes, {
      path
    })
  };
}
/**
 * sort vinyl stream by filename
 * + it sort to be end of sorting, if filename is like "pagelist".
 * @param {Object} options options
 * @param {String} options.extname extname
 * @return {Number}
 */


function sortByFilename(options = {}) {
  const extname = options.extname || '.html',
        pattern = new RegExp(`page-?list${extname}$`);
  return (a, b) => {
    const aIsMatch = pattern.test(a.path),
          bIsMatch = pattern.test(b.path);

    if (aIsMatch && !bIsMatch) {
      return 1; // eslint-disable-line no-magic-numbers
    } else if (!aIsMatch && bIsMatch) {
      return -1; // eslint-disable-line no-magic-numbers
    }

    return a.path.localeCompare(b.path);
  };
}
/**
 * return html build task by handlebars
 * @param  {DEFAULT_OPTIONS} options options
 * @return {Function<Promise>}
 *
 * @example
 * import {task} from 'gulp';
 * import buildHtml from '@hidoo/gulp-task-build-html-handlebars';
 *
 * task('html', buildHtml({
 *   src: '/path/to/html/*.hbs',
 *   dest: '/path/to/dest'
 * }));
 */


function buildHtml(options = {}) {
  const opts = _objectSpread({}, DEFAULT_OPTIONS, options); // define task


  const task = () => {
    const {
      extname,
      compress,
      compressOptions,
      onFilesParsed,
      onFrontMatterParsed,
      verbose
    } = opts,
          // eslint-disable-line max-len
    handlebars = _handlebars.default.create(),
          context = getContextFromFiles(opts.data, {
      onParsed: onFilesParsed,
      handlebars,
      verbose
    }),
          // eslint-disable-line max-len
    pages = [],
          addtionalContext = {
      NODE_ENV: process.env.NODE_ENV || 'development',
      // eslint-disable-line no-process-env
      compress,
      pages
    }; // add default helpers from @hidoo/handlebars-helpers


    Object.entries(helpers).forEach(([name, helper]) => handlebars.registerHelper(name, helper));
    return (0, _gulp.src)(opts.src).pipe((0, _gulpPlumber.default)({
      errorHandler: _gulpUtilErrorHandler.default
    })).pipe((0, _gulpRename.default)({
      extname
    })).pipe((0, _gulpSort.default)(sortByFilename({
      extname
    }))).pipe((0, _gulpData.default)(file => {
      const {
        body,
        attributes
      } = getContextFromFrontMatter(file, {
        context,
        handlebars,
        onParsed: onFrontMatterParsed
      });
      file.contents = Buffer.from(body);
      pages.push(attributes);
      return (0, _lodash.default)(attributes);
    })).pipe((0, _gulpHb.default)({
      handlebars
    }).helpers(_handlebarsLayouts.default).helpers(opts.helpers).partials(opts.layouts).partials(opts.partials).data(context).data(addtionalContext)).pipe((0, _gulp.dest)(opts.dest)).pipe((0, _gulpIf.default)(compress, (0, _gulpHtmlmin.default)(_objectSpread({}, compressOptions)))).pipe((0, _gulpIf.default)(compress, (0, _gulp.dest)(opts.dest)));
  }; // add displayName (used as task name for gulp)


  if (typeof opts.name === 'string' && opts.name !== '') {
    task.displayName = opts.name;
  }

  return task;
}